import 'package:app_review_helper/src/models/review_dialog_config.dart';
import 'package:app_review_helper/src/models/review_state.dart';
import 'package:conditional_trigger/conditional_trigger.dart';
import 'package:flutter/foundation.dart';
import 'package:in_app_review/in_app_review.dart';
import 'package:update_helper/update_helper.dart';
import 'package:url_launcher/url_launcher_string.dart';

import 'models/review_mock.dart';

class AppReviewHelper {
  /// Get AppReviewHelper instance.
  static AppReviewHelper instance = AppReviewHelper._internal();
  static ReviewMock? _mock;

  /// Set mock values.
  static void setMockInitialValues([ReviewMock? mock]) {
    _mock = mock;
  }

  AppReviewHelper._internal();

  /// Debug
  bool _isDebug = false;

  /// Open the store if available, if not, it'll try opening the `fallbackUrl`.
  Future<void> openStore({String? fallbackUrl, bool debugLog = false}) async {
    if (kIsWeb) {
      if (fallbackUrl != null && await canLaunchUrlString(fallbackUrl)) {
        _print('Open the fallbackUrl on Web platform: $fallbackUrl');
        await launchUrlString(fallbackUrl);
      } else {
        _print('Web platform and fallbackUrl == null');
      }

      return;
    }

    await UpdateHelper.openStore(fallbackUrl: fallbackUrl, debugLog: debugLog);
  }

  /// This function will request an in-app review every time a new version is published
  /// and it satisfy with the conditions.
  Future<ReviewState> initial({
    /// Show the dialog with thump up - down to let the user to choose before
    /// requesting a review. Only request a review if the thump up is chosen.
    /// If not, the dialog with text field will be shown to get the review
    /// from user.
    ReviewDialogConfig? reviewDialogConfig,

    /// Min days
    int minDays = 3,

    /// If you add this line in your main(), it's same as app opening count
    int minCalls = 3,

    /// If the current version is satisfied with this than not showing the request
    /// this value use plugin `satisfied_version` to compare.
    List<String> noRequestVersions = const [],

    /// List of version that allow the app to remind the in-app review.
    List<String> remindedVersions = const [],

    /// If true, it'll keep asking for the review on each new version (and satisfy with all the above conditions).
    /// If false, it only requests for the first time the conditions are satisfied.
    bool keepRemind = false,

    /// Request with delayed duaration
    Duration? duration,

    /// Print debug log
    bool isDebug = !kReleaseMode,
  }) async {
    _isDebug = isDebug;

    final supportedPlatform =
        (defaultTargetPlatform == TargetPlatform.android ||
                defaultTargetPlatform == TargetPlatform.iOS) &&
            !kIsWeb;
    if (!supportedPlatform) {
      return _print(ReviewState.unSupportedPlatform)!;
    }

    final isAvailable = _mock != null
        ? _mock?.inAppReviewForceState == true
        : await InAppReview.instance.isAvailable();

    if (!isAvailable) {
      return _print(ReviewState.unavailable)!;
    }

    final condition = ConditionalTrigger(
      'AppReviewHelper',
      minDays: minDays,
      minCalls: minCalls,
      noRequestVersions: noRequestVersions,
      remindedVersions: remindedVersions,
      keepRemind: keepRemind,
      debugLog: isDebug,
    );

    condition.setMockInitialValues(_mock);

    switch (await condition.check()) {
      case ConditionalState.keepRemindDisabled:
        return _print(ReviewState.keepRemindDisabled)!;
      case ConditionalState.noRequestVersion:
        return _print(ReviewState.noRequestVersion)!;
      case ConditionalState.notSatisfiedWithMinCallsAndDays:
        return _print(ReviewState.notSatisfiedWithMinCallsAndDays)!;
      case ConditionalState.notSatisfiedWithMinCalls:
        return _print(ReviewState.notSatisfiedWithMinCalls)!;
      case ConditionalState.notSatisfiedWithMinDays:
        return _print(ReviewState.notSatisfiedWithMinDays)!;
      case ConditionalState.satisfied:
        if (!isDebug) {
          if (duration != null) await Future.delayed(duration);
          return _print(ReviewState.completed)!;
        } else {
          return _print(ReviewState.compeletedInDebugMode)!;
        }
    }
  }

  ReviewState? _print(Object log) {
    if (log is ReviewState) {
      if (_isDebug) {
        debugPrint('[App Review Helper] ${log.text}');
      }
      return log;
    }
    if (_isDebug) {
      debugPrint('[App Review Helper] $log');
    }
    return null;
  }
}
